package example.localplugin;

import gridsim.dcworms.DCWormsTags;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import schedframe.events.scheduling.SchedulingEvent;
import schedframe.resources.ResourceStatus;
import schedframe.resources.computing.Node;
import schedframe.resources.computing.ComputingResource;
import schedframe.resources.units.Memory;
import schedframe.resources.units.ProcessingElements;
import schedframe.resources.units.ResourceUnit;
import schedframe.resources.units.ResourceUnitName;
import schedframe.resources.units.StandardResourceUnitName;
import schedframe.scheduling.manager.resources.ClusterResourceManager;
import schedframe.scheduling.manager.resources.ResourceManager;
import schedframe.scheduling.manager.tasks.JobRegistry;
import schedframe.scheduling.plan.SchedulingPlanInterface;
import schedframe.scheduling.plan.impl.SchedulingPlan;
import schedframe.scheduling.plugin.grid.ModuleList;
import schedframe.scheduling.queue.TaskQueue;
import schedframe.scheduling.queue.TaskQueueList;
import schedframe.scheduling.tasks.TaskInterface;

public class Cluster_FCFSBF_ConsolidationPlugin extends BaseLocalSchedulingPlugin {

	public Cluster_FCFSBF_ConsolidationPlugin () {
	}

	public SchedulingPlanInterface<?> schedule(SchedulingEvent event, TaskQueueList queues, JobRegistry jobRegistry,
			ResourceManager resManager, ModuleList modules) {

		ClusterResourceManager resourceManager = (ClusterResourceManager) resManager;
		SchedulingPlan plan = new SchedulingPlan();
		// choose the events types to serve.
		// Different actions for different events are possible.
		switch (event.getType()) {
		case START_TASK_EXECUTION:
		case TASK_FINISHED:
			// our tasks are placed only in first queue (see
			// BaseLocalSchedulingPlugin.placeJobsInQueues() method)
			TaskQueue q = queues.get(0);
			// check all tasks in queue

			for (int i = 0; i < q.size(); i++) {
				TaskInterface<?> task = q.get(i);
				// if status of the tasks in READY
				if (task.getStatus() == DCWormsTags.READY) {

					Map<ResourceUnitName, ResourceUnit> choosenResources =  chooseResourcesForExecution(resourceManager, task);
					if (choosenResources  != null) {
						addToSchedulingPlan(plan, task, choosenResources);
					} 
				}
			}

			break;
		}
		return plan;
	}
	
	private Map<ResourceUnitName, ResourceUnit> chooseResourcesForExecution(ClusterResourceManager resourceManager, TaskInterface<?> task){

		List<Node> nodes = resourceManager.getNodes();
		nodes = findSuitableNodes(task, nodes);
		Collections.sort(nodes, new Comparator<Node>(){
		    public int compare(Node node1, Node node2){   
		        return node1.getResourceCharacteristic().getParameters().get("category").get(0).getContent().compareTo(node2.getResourceCharacteristic().getParameters().get("category").get(0).getContent());
		    }
		});
		if(nodes.size() > 0)
		{
			Node node = nodes.get(0);
			Map<ResourceUnitName, ResourceUnit> map = new HashMap<ResourceUnitName, ResourceUnit>();
			List<ComputingResource> choosenResources =  new ArrayList<ComputingResource>();
			int cpuRequest;
			try {
				cpuRequest = Double.valueOf(task.getCpuCntRequest()).intValue();
			} catch (NoSuchFieldException e) {
				cpuRequest = 0;
			}
			for (int i = 0; i < node.getProcessors().size() && cpuRequest > 0; i++) {
				if (node.getProcessors().get(i).getStatus() == ResourceStatus.FREE) {
					choosenResources.add(node.getProcessors().get(i));
					cpuRequest--;
				}
			}
			ProcessingElements result = new ProcessingElements(node.getFullName());
			result.addAll(choosenResources);
			map.put(StandardResourceUnitName.PE, result);
			
			int memoryRequest;
			try {
				memoryRequest = Double.valueOf(task.getMemoryRequest()).intValue();
			} catch (NoSuchFieldException e) {
				memoryRequest = 0;
			}
			if (memoryRequest != 0) {
				Memory memory;
				try {
					memory = new Memory(node.getMemory(), memoryRequest, memoryRequest);
					map.put(StandardResourceUnitName.MEMORY, memory);
				} catch (NoSuchFieldException e) {
					return null;
				}

			}
			return map;
		} else 
			return null;
	}
	
	private List<Node> findSuitableNodes(TaskInterface<?> task, List<Node> nodes) {
		int cpuRequest;
		try {
			cpuRequest = Double.valueOf(task.getCpuCntRequest()).intValue();
		} catch (NoSuchFieldException e) {
			cpuRequest = 1;
		}
		int memoryRequest;
		try {
			memoryRequest = Double.valueOf(task.getMemoryRequest()).intValue();
		} catch (NoSuchFieldException e) {
			memoryRequest = 0;
		}
		List<Node> suitableNodes = new ArrayList<Node>();
		for(Node node: nodes){
			try{
				if(node.getFreeProcessorsNumber() >= cpuRequest && node.getFreeMemory() >= memoryRequest){
					suitableNodes.add(node);
				}	
			} catch (NoSuchFieldException e){
				
			}
		}
		return suitableNodes;
	}

}
